/**
 * @author rik
 */
var gulp = require('gulp');
var _ = require('lodash');
var gulpLoadPlugins = require('gulp-load-plugins');
var path = require('path');
var includeAll = require('include-all');

var growl = false;

var defaultTaskConfigurationDir = './tasks/register';
var defaultRegisterDir = './tasks/config';

/**
 * Reads all files from a config and register directory, each file exporting a function that gets called with gulp and plugins
 * @global taskrunner
 * @param options
 */
function taskrunner(options) {
  options = options || {};
  options.gulpLoadPluginsOptions = options.gulpLoadPluginsOptions || {};

  _.defaults(options.gulpLoadPluginsOptions, {
    pattern: ['gulp-*', 'merge-*', 'run-*', 'main-*'], // the glob to search for
    replaceString: /\bgulp[\-.]|run[\-.]|merge[\-.]|main[\-.]/, // what to remove from the name of the module when adding it to the context
    camelizePluginName: true, // if true, transforms hyphenated plugins names to camel case
    config: path.resolve(process.cwd(), './package.json'),
    lazy: true // whether the plugins should be lazy loaded on demand
  });

  // gets the files from the config and register directory, they are treated equally
  var taskConfigurations = loadTasks(options.taskConfigurationDir || defaultTaskConfigurationDir);
  var registerDefinitions = loadTasks(options.registerDir || defaultRegisterDir);

  var plugins = gulpLoadPlugins(options.gulpLoadPluginsOptions);

  invokeConfigFn(taskConfigurations, plugins);
  invokeConfigFn(registerDefinitions, plugins);
}

function ensureDefaultTask(registerDefinitions) {
  registerDefinitions.default = registerDefinitions.default || defaultTask;
}

var defaultTask = function (gulp) {
  gulp.task('default', []);
};

function loadTasks(relPath) {
  return includeAll({
      dirname: path.resolve(process.cwd(), relPath),
      filter: /(.+)\.js$/i
    }) || {};
}

function invokeConfigFn(tasks, plugins) {
  _.each(tasks, function (task) {
    task(gulp, plugins, growl, path);
  });
}

module.exports = taskrunner;